"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var check = function (_a, value, genericsMap, checkerMap) {
    var type = _a.type, data = _a.data;
    switch (type) {
        case 'literal':
            return value === data;
        case 'primitive': {
            if (data === 'any' && value !== null
                || data === 'void' && value === undefined)
                return true;
            var valueType = typeof value;
            return data === 'bool' && valueType === 'boolean'
                || data === 'char' && valueType === 'string' && value.length === 1
                || data === 'float' && valueType === 'number'
                || data === 'int' && valueType === 'number' && (value | 0) === value
                || data === 'string' && valueType === 'string';
        }
        case 'generic': {
            var _b = data, extendsType = _b.extends, name_1 = _b.name;
            if (genericsMap) {
                var generic = genericsMap[name_1];
                if (generic)
                    return check(generic, value, genericsMap, checkerMap);
            }
            return !extendsType || check(extendsType, value, genericsMap, checkerMap);
        }
        case 'tuple': {
            if (!Array.isArray(value))
                return false;
            var length_1 = value.length;
            return data ?
                length_1 === data.length
                    && !~data.findIndex(function (currentType, index) {
                        return !check(currentType, value[index], genericsMap, checkerMap);
                    })
                : !length_1;
        }
        case 'array': {
            if (!Array.isArray(value))
                return false;
            var size = data.size, valueType_1 = data.type;
            return value.length === size
                && !~value.findIndex(function (currentValue) {
                    return !check(valueType_1, currentValue, genericsMap, checkerMap);
                });
        }
        case 'list':
            if (!Array.isArray(value))
                return false;
            return !~value.findIndex(function (currentValue) {
                return !check(data, currentValue, genericsMap, checkerMap);
            });
        case 'struct': {
            if (typeof value !== 'object' || Array.isArray(value))
                return false;
            return !data
                || !~Object.keys(data).findIndex(function (key) {
                    var pair = data[key];
                    var currentValue = value[key];
                    return !(pair.optional && currentValue === undefined
                        || check(pair.type, currentValue, genericsMap, checkerMap));
                });
        }
        case 'map': {
            if (typeof value !== 'object' || Array.isArray(value))
                return false;
            var _c = data, keyType_1 = _c.keyType, valueType_2 = _c.type;
            return !~Object.keys(value).findIndex(function (key) {
                return !(check(keyType_1, key, genericsMap, checkerMap)
                    && check(valueType_2, value[key], genericsMap, checkerMap));
            });
        }
        case 'named': {
            var valueType = data.type;
            if (valueType && !check(valueType, value, genericsMap, checkerMap))
                return false;
            if (checkerMap) {
                var checker = checkerMap[data.name];
                return !checker || checker(value);
            }
            return true;
        }
        case 'nullable':
            return value === null
                || check(data, value, genericsMap, checkerMap);
        case 'intersection':
            return check(data.type, value, genericsMap, checkerMap)
                && check(data.type2, value, genericsMap, checkerMap);
        case 'union':
            return check(data.type, value, genericsMap, checkerMap)
                || check(data.type2, value, genericsMap, checkerMap);
    }
    return false;
};
exports.default = check;
//# sourceMappingURL=check.js.map