"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var check_1 = require("./check");
var grammar_1 = require("./grammar");
var utils_1 = require("./utils");
var nullableAny = new grammar_1.default('nullable', new grammar_1.default('primitive', 'any'));
var match = function (receivingType, receivedType, genericsMap, context) {
    if (context === void 0) { context = { readonlyGenerics: true }; }
    var data = receivingType.data, type = receivingType.type;
    var data2 = receivedType.data, type2 = receivedType.type;
    if (type2 === 'nullable') {
        if (data2.type === 'union') {
            var subData = data2.data;
            return match(receivingType, new grammar_1.default('nullable', subData.type), genericsMap, context)
                && match(receivingType, new grammar_1.default('nullable', subData.type2), genericsMap, context);
        }
        else if (data2.type === 'intersection') {
            var subData = data2.data;
            return match(receivingType, new grammar_1.default('nullable', subData.type), genericsMap, context)
                || match(receivingType, new grammar_1.default('nullable', subData.type2), genericsMap, context);
        }
    }
    else if (type2 === 'union') {
        return match(receivingType, data2.type, genericsMap, context)
            && match(receivingType, data2.type2, genericsMap, context);
    }
    switch (type) {
        case 'primitive':
            if (data === 'any'
                && !(type2 === 'literal' && data2 === null)
                && type2 !== 'generic'
                && type2 !== 'named'
                && type2 !== 'nullable'
                && type2 !== 'intersection')
                return true;
            break;
        case 'generic':
            if (genericsMap) {
                var generic = genericsMap[data.name];
                if (context.readonlyGenerics) {
                    if (generic)
                        return match(generic, receivedType, genericsMap, context);
                }
                else {
                    if (data.extends
                        && !match(data.extends, receivedType, genericsMap, context))
                        return false;
                    if (generic) {
                        if (!match(generic, receivedType, genericsMap, context)) {
                            genericsMap[data.name] = new grammar_1.default('union', {
                                type: generic,
                                type2: receivedType,
                            });
                        }
                    }
                    else {
                        genericsMap[data.name] = receivedType;
                    }
                    return true;
                }
            }
            return !data.extends || match(data.extends, receivedType, genericsMap, context);
        case 'nullable':
            return match(data, receivedType, genericsMap, __assign({}, context, { nullable: true }));
        case 'intersection':
            return match(data.type, receivedType, genericsMap, context)
                && match(data.type2, receivedType, genericsMap, context);
        case 'union':
            var newContext = utils_1.isNullable(data.type) || utils_1.isNullable(data.type2) ? __assign({}, context, { nullable: true }) : context;
            return match(data.type, receivedType, genericsMap, newContext)
                || match(data.type2, receivedType, genericsMap, newContext);
    }
    switch (type2) {
        case 'literal':
            return type === type2 ? data === data2
                : data2 === null && context.nullable
                    || check_1.default(receivingType, data2, genericsMap);
        case 'primitive':
            return type === type2 && data === data2;
        case 'generic':
            return data2.extends ? match(receivingType, data2.extends, genericsMap, context)
                : match(receivingType, nullableAny, genericsMap, context);
        case 'tuple':
            return type === type2
                && (data === data2
                    || data && data2 && data.length === data2.length &&
                        !~data.findIndex(function (valueType, index) {
                            var valueType2 = data2[index];
                            return !(valueType2 && match(valueType, valueType2));
                        }));
        case 'array':
            return type === type2
                && data.size === data2.size
                && match(data.type, data2.type, genericsMap);
        case 'list':
            return type === type2
                && match(data, data2, genericsMap);
        case 'struct':
            return type === type2
                && (data === null
                    || data2 !== null && !~Object.keys(data).findIndex(function (key) {
                        var pair = data[key];
                        var pair2 = data2[key];
                        return !(pair2 ?
                            (pair.optional || !pair2.optional)
                                && match(pair.type, pair2.type, genericsMap, context)
                            : pair.optional);
                    }));
        case 'map':
            return type === type2
                && match(data.keyType, data2.keyType, genericsMap)
                && match(data.type, data2.type, genericsMap);
        case 'named':
            return type === type2 ?
                (data.name === data2.name
                    && (data.type === data2.type
                        || data.type && data2.type && match(data.type, data2.type, genericsMap, context)))
                : match(receivingType, data2.type, genericsMap, context);
        case 'nullable':
            return Boolean(context.nullable
                && match(receivingType, data2, genericsMap, context));
        case 'intersection': {
            var newContext = context;
            if (!context.nullable) {
                var null1 = utils_1.isNullable(data2.type);
                var null2 = utils_1.isNullable(data2.type2);
                if (!(null1 && null2) && (null1 || null2))
                    newContext = __assign({}, context, { nullable: true });
            }
            return match(receivingType, data2.type, genericsMap, newContext)
                || match(receivingType, data2.type2, genericsMap, newContext);
        }
    }
    return false;
};
exports.default = match;
//# sourceMappingURL=match.js.map