"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureParentheses = function (typeAST) {
    var type = typeAST.type;
    return type === 'union' || type === 'intersection' || type === 'nullable' ?
        "(" + print(typeAST) + ")"
        : print(typeAST);
};
var print = function (_a) {
    var type = _a.type, data = _a.data;
    switch (type) {
        case 'literal':
            return JSON.stringify(data);
        case 'primitive':
            return data;
        case 'generic': {
            var extendsType = data.extends;
            return "" + data.name + (extendsType ? " extends " + exports.ensureParentheses(extendsType) : '');
        }
        case 'tuple':
            return data ? "[" + data.map(print).join(', ') + "]" : '[]';
        case 'array':
            return exports.ensureParentheses(data.type) + "[" + data.size + "]";
        case 'list':
            return exports.ensureParentheses(data) + "[]";
        case 'struct':
            return data ? "{ " + Object.keys(data).map(function (key) {
                return "" + key + (data[key].optional ? '?' : '') + ": " + print(data[key].type);
            }).join(', ') + " }" : '{}';
        case 'map':
            return "{ [" + print(data.keyType) + "]: " + print(data.type) + " }";
        case 'named':
            return data.name + "{" + (data.type ? print(data.type) : '') + "}";
        case 'nullable':
            return "?" + exports.ensureParentheses(data);
        case 'intersection':
            return print(data.type) + " & " + print(data.type2);
        case 'union':
            return print(data.type) + " | " + print(data.type2);
    }
    return '?any';
};
exports.default = print;
//# sourceMappingURL=print.js.map